#!/bin/bash

# This file is part of Marionnet, a virtual network laboratory
# Copyright (C) 2023  Jean-Vincent Loddo
# Copyright (C) 2023  Université Sorbonne Paris Nord (USPN)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ---
SRC_PROJECT_DIR="$(dirname $0)/.."
# ---
SOURCE1=${1:-"$SRC_PROJECT_DIR/META"}
SOURCE2=${2:-"$SRC_PROJECT_DIR/CONFIGME"}
TARGET=${3:-"$SRC_PROJECT_DIR/bin/meta.ml"}
# ---
# If the file "meta.ml.released" exists, "meta.ml" is just a copy:
RELEASED="$TARGET.released"
if test -f $RELEASED; then
  cp -f $RELEASED $TARGET || exit 4
  exit 0
fi
# else continue:
# ---
# Example of the content of META:
# ---
# name="marionnet"
# description="A virtual network laboratory"
# version="trunk"
# requires="threads str unix lablgtk3 lablglade ocamlbricks"
# ---
# Some defaults:
version="trunk"
prefix="/usr/local"
configurationprefix="/etc"
localeprefix="/usr/local/share/locale"
documentationprefix="/usr/local/share/doc"
# ---
if ! test -f $SOURCE1; then echo 1>&2 "Error: file $SOURCE1 not found => Exiting..."; exit 1; fi
if ! test -f $SOURCE2; then echo 1>&2 "Error: file $SOURCE2 not found => Exiting..."; exit 2; fi
# ---
source $SOURCE1 && source $SOURCE2 || exit 3
# ---
if [[ -d "$SRC_PROJECT_DIR/.bzr" ]]; then
  revision="$(bzr revno)"
  source_date="$(bzr info --verbose | /bin/grep 'latest revision' | cut -d: -f2- | cut -d' ' -f3-)"
  DATE="$(bzr log -r $revision --timezone utc | awk '/^timestamp/' | cut -f2- -d:)"
  source_date_utc_yy_mm_dd=$(date -d "$DATE" -u '+%Y-%m-%d')
else
  echo 1>&2 "Warning: directory $SRC_PROJECT_DIR/.bzr not found";
fi
# ---
cat >$TARGET <<EOF
(* --- *)
(* Automatically generated meta-informations about the project and its building. *)
(* This file is automatically generated; please don't edit it. *)
(* --- *)
let name = "$name";;
let version = "$version";;
let prefix = "$prefix";;
let prefix_install = "$prefix_install";;
let ocaml_version = "$ocaml_version";;
let ocaml_libraryprefix = "$ocaml_libraryprefix";;
let libraryprefix = "$libraryprefix";;
let configurationprefix = "$configurationprefix";;
let localeprefix = "$localeprefix";;
let documentationprefix = "$documentationprefix";;
let uname = "$(uname -srvmo)";;
let build_date = "$(date '+%Y-%m-%d %k:%M:%S %z')";;
let revision = "$revision";;
let source_date = "$source_date";;
let source_date_utc_yy_mm_dd = "$source_date_utc_yy_mm_dd";;
EOF
# ---
echo "Success." 1<&2
